==> glowroot-test-harness-0.8.3-sources.jar: org/glowroot/weaving/AdviceBase.java <==
/*
 * Copyright 2012-2015 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.glowroot.weaving;

import java.util.List;
import java.util.Set;
import java.util.regex.Pattern;

import javax.annotation.Nullable;

import org.glowroot.shaded.google.common.collect.ImmutableList;
import org.glowroot.shaded.google.common.collect.ImmutableSet;
import org.glowroot.shaded.google.common.collect.Ordering;
import org.glowroot.shaded.google.common.collect.Sets;
import org.glowroot.shaded.google.common.primitives.Ints;
import org.immutables.value.Value;
import org.glowroot.shaded.objectweb.asm.Type;
import org.glowroot.shaded.objectweb.asm.commons.Method;

import org.glowroot.api.weaving.Pointcut;

import static org.glowroot.shaded.google.common.base.Preconditions.checkNotNull;

@Value.Immutable
public abstract class AdviceBase {

    static final Ordering<Advice> ordering = new AdviceOrdering();

    abstract Pointcut pointcut();
    abstract Type adviceType();
    abstract @Nullable Pattern pointcutClassNamePattern();
    abstract @Nullable Pattern pointcutMethodNamePattern();
    abstract @Nullable Type travelerType();
    abstract @Nullable Method isEnabledAdvice();
    abstract @Nullable Method onBeforeAdvice();
    abstract @Nullable Method onReturnAdvice();
    abstract @Nullable Method onThrowAdvice();
    abstract @Nullable Method onAfterAdvice();
    abstract ImmutableList<AdviceParameter> isEnabledParameters();
    abstract ImmutableList<AdviceParameter> onBeforeParameters();
    abstract ImmutableList<AdviceParameter> onReturnParameters();
    abstract ImmutableList<AdviceParameter> onThrowParameters();
    abstract ImmutableList<AdviceParameter> onAfterParameters();
    abstract boolean reweavable();

    @Value.Derived
    ImmutableSet<Type> classMetaTypes() {
        Set<Type> metaTypes = Sets.newHashSet();
        metaTypes.addAll(getClassMetaTypes(isEnabledParameters()));
        metaTypes.addAll(getClassMetaTypes(onBeforeParameters()));
        metaTypes.addAll(getClassMetaTypes(onReturnParameters()));
        metaTypes.addAll(getClassMetaTypes(onThrowParameters()));
        metaTypes.addAll(getClassMetaTypes(onAfterParameters()));
        return ImmutableSet.copyOf(metaTypes);
    }

    @Value.Derived
    ImmutableSet<Type> methodMetaTypes() {
        Set<Type> metaTypes = Sets.newHashSet();
        metaTypes.addAll(getMethodMetaTypes(isEnabledParameters()));
        metaTypes.addAll(getMethodMetaTypes(onBeforeParameters()));
        metaTypes.addAll(getMethodMetaTypes(onReturnParameters()));
        metaTypes.addAll(getMethodMetaTypes(onThrowParameters()));
        metaTypes.addAll(getMethodMetaTypes(onAfterParameters()));
        return ImmutableSet.copyOf(metaTypes);
    }

    private static Set<Type> getClassMetaTypes(List<AdviceParameter> parameters) {
        Set<Type> types = Sets.newHashSet();
        for (AdviceParameter parameter : parameters) {
            if (parameter.kind() == ParameterKind.CLASS_META) {
                types.add(parameter.type());
            }
        }
        return types;
    }

    private static Set<Type> getMethodMetaTypes(List<AdviceParameter> parameters) {
        Set<Type> types = Sets.newHashSet();
        for (AdviceParameter parameter : parameters) {
            if (parameter.kind() == ParameterKind.METHOD_META) {
                types.add(parameter.type());
            }
        }
        return types;
    }

    private static final class AdviceOrdering extends Ordering<Advice> {
        @Override
        public int compare(@Nullable Advice left, @Nullable Advice right) {
            checkNotNull(left);
            checkNotNull(right);
            int compare = Ints.compare(left.pointcut().priority(), right.pointcut().priority());
            if (compare != 0) {
                return compare;
            }
            String leftTimerName = left.pointcut().timerName();
            String rightTimerName = right.pointcut().timerName();
            // empty timer names are placed at the end
            if (leftTimerName.isEmpty() && rightTimerName.isEmpty()) {
                return 0;
            }
            if (leftTimerName.isEmpty()) {
                return 1;
            }
            if (rightTimerName.isEmpty()) {
                return -1;
            }
            return leftTimerName.compareToIgnoreCase(rightTimerName);
        }
    }

    @Value.Immutable
    abstract static class AdviceParameterBase {
        abstract ParameterKind kind();
        abstract Type type();
    }
}

==> glowroot-test-harness-0.8.4-sources.jar: org/glowroot/weaving/AdviceBase.java <==
/*
 * Copyright 2012-2015 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.glowroot.weaving;

import java.util.List;
import java.util.Set;
import java.util.regex.Pattern;

import javax.annotation.Nullable;

import org.glowroot.shaded.google.common.collect.ImmutableList;
import org.glowroot.shaded.google.common.collect.ImmutableSet;
import org.glowroot.shaded.google.common.collect.Ordering;
import org.glowroot.shaded.google.common.collect.Sets;
import org.glowroot.shaded.google.common.primitives.Ints;
import org.immutables.value.Value;
import org.glowroot.shaded.objectweb.asm.Type;
import org.glowroot.shaded.objectweb.asm.commons.Method;

import org.glowroot.plugin.api.weaving.Pointcut;

import static org.glowroot.shaded.google.common.base.Preconditions.checkNotNull;

@Value.Immutable
public abstract class AdviceBase {

    static final Ordering<Advice> ordering = new AdviceOrdering();

    abstract Pointcut pointcut();
    abstract Type adviceType();
    abstract String pointcutDeclaringClassName();
    abstract @Nullable String pointcutTargetClassName();
    abstract @Nullable Pattern pointcutDeclaringClassNamePattern();
    abstract @Nullable Pattern pointcutTargetClassNamePattern();
    abstract @Nullable Pattern pointcutMethodNamePattern();
    abstract @Nullable Type travelerType();
    abstract @Nullable Method isEnabledAdvice();
    abstract @Nullable Method onBeforeAdvice();
    abstract @Nullable Method onReturnAdvice();
    abstract @Nullable Method onThrowAdvice();
    abstract @Nullable Method onAfterAdvice();
    abstract ImmutableList<AdviceParameter> isEnabledParameters();
    abstract ImmutableList<AdviceParameter> onBeforeParameters();
    abstract ImmutableList<AdviceParameter> onReturnParameters();
    abstract ImmutableList<AdviceParameter> onThrowParameters();
    abstract ImmutableList<AdviceParameter> onAfterParameters();
    abstract boolean reweavable();

    @Value.Derived
    ImmutableSet<Type> classMetaTypes() {
        Set<Type> metaTypes = Sets.newHashSet();
        metaTypes.addAll(getClassMetaTypes(isEnabledParameters()));
        metaTypes.addAll(getClassMetaTypes(onBeforeParameters()));
        metaTypes.addAll(getClassMetaTypes(onReturnParameters()));
        metaTypes.addAll(getClassMetaTypes(onThrowParameters()));
        metaTypes.addAll(getClassMetaTypes(onAfterParameters()));
        return ImmutableSet.copyOf(metaTypes);
    }

    @Value.Derived
    ImmutableSet<Type> methodMetaTypes() {
        Set<Type> metaTypes = Sets.newHashSet();
        metaTypes.addAll(getMethodMetaTypes(isEnabledParameters()));
        metaTypes.addAll(getMethodMetaTypes(onBeforeParameters()));
        metaTypes.addAll(getMethodMetaTypes(onReturnParameters()));
        metaTypes.addAll(getMethodMetaTypes(onThrowParameters()));
        metaTypes.addAll(getMethodMetaTypes(onAfterParameters()));
        return ImmutableSet.copyOf(metaTypes);
    }

    private static Set<Type> getClassMetaTypes(List<AdviceParameter> parameters) {
        Set<Type> types = Sets.newHashSet();
        for (AdviceParameter parameter : parameters) {
            if (parameter.kind() == ParameterKind.CLASS_META) {
                types.add(parameter.type());
            }
        }
        return types;
    }

    private static Set<Type> getMethodMetaTypes(List<AdviceParameter> parameters) {
        Set<Type> types = Sets.newHashSet();
        for (AdviceParameter parameter : parameters) {
            if (parameter.kind() == ParameterKind.METHOD_META) {
                types.add(parameter.type());
            }
        }
        return types;
    }

    private static final class AdviceOrdering extends Ordering<Advice> {
        @Override
        public int compare(@Nullable Advice left, @Nullable Advice right) {
            checkNotNull(left);
            checkNotNull(right);
            int compare = Ints.compare(left.pointcut().priority(), right.pointcut().priority());
            if (compare != 0) {
                return compare;
            }
            String leftTimerName = left.pointcut().timerName();
            String rightTimerName = right.pointcut().timerName();
            // empty timer names are placed at the end
            if (leftTimerName.isEmpty() && rightTimerName.isEmpty()) {
                return 0;
            }
            if (leftTimerName.isEmpty()) {
                return 1;
            }
            if (rightTimerName.isEmpty()) {
                return -1;
            }
            return leftTimerName.compareToIgnoreCase(rightTimerName);
        }
    }

    @Value.Immutable
    abstract static class AdviceParameterBase {
        abstract ParameterKind kind();
        abstract Type type();
    }
}
